/* Project 8 by David Taralla - PART 1 | 18-04-2010
*  =================================================
*  This is a little reversi game, respecting Othello rules.
*  The program use rvGrid & rvPlayer interfaces
*  (see rvGrid.h/rvGrid.c and rvPlayer.h/rvPlayer.c).
*/

#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include "rvGrid.h"
#include "rvPlayer.h"


/******************************************************************************
*------------------------------- Main program --------------------------------*
******************************************************************************/
// Possible args : [width height [player-port [grid-host grid-port]]]
int main(int argc, char* argv[]) {
    // -- PREPAIRING GAME --
    
    // - Creating grid -
    rvGrid* grid = NULL;
    
    // If not enough args, height = width = 8
    if (argc < 3)
        grid = rvGridNew("localhost", 7777, 8, 8);
    else
        grid = rvGridNew(argv[4], atoi(argv[5]), atoi(argv[1]), atoi(argv[2]));
    
    /* - Instancing players -
       -> player1 is BLACK and player2 is WHITE */
    rvPlayer* player1 = rvPlayerNew(atoi(argv[3]), atoi(argv[1]),
                                    atoi(argv[2]));
    rvPlayer* player2 = rvPlayerNew(atoi(argv[3]), atoi(argv[1]),
                                    atoi(argv[2]));
    

    // -- STARTING GAME --
    while (true) {
        int winMove, activeRow, activeCol;


        // -- BLACK'S TURN --
        if (canPlay(grid, BLACK)) {
            rvGridDisplay(grid, rvPlayerOutput(player1));
            rvPlayerAddDisk(player1, &activeRow, &activeCol);

            winMove = rvGridAddDisk(grid, BLACK, activeRow, activeCol);
            while (winMove < 0) {
                rvPlayerAddDisk(player1, &activeRow, &activeCol);
                winMove = rvGridAddDisk(grid, BLACK, activeRow, activeCol);
            }

            // If it was a terminal move, stopping game
            if (winMove != 0) {
                // Player 1 (BLACK) has won
                if (winMove == 1) {
                    rvPlayerPrintResult(player1, 0);
                    rvPlayerPrintResult(player2, 1);
                }
                // Player 1 (BLACK) has lost
                else if (winMove == 2) {
                    rvPlayerPrintResult(player1, 1);
                    rvPlayerPrintResult(player2, 0);
                }
                // Draw
                else {
                    rvPlayerPrintResult(player1, 2);
                    rvPlayerPrintResult(player2, 2);
                }

                break;
            }
        }


        // -- WHITE'S TURN --
        if (canPlay(grid, WHITE)) {
            rvGridDisplay(grid, rvPlayerOutput(player2));
            rvPlayerAddDisk(player2, &activeRow, &activeCol);

            winMove = rvGridAddDisk(grid, WHITE, activeRow, activeCol);
            while (winMove < 0) {
                rvPlayerAddDisk(player2, &activeRow, &activeCol);
                winMove = rvGridAddDisk(grid, WHITE, activeRow, activeCol);
            }

            // If it was a terminal move, stopping game
            if (winMove != 0) {
                // Player 2 (WHITE) has won
                if (winMove == 1) {
                    rvPlayerPrintResult(player2, 0);
                    rvPlayerPrintResult(player1, 1);
                }
                // Player 2 (WHITE) has lost
                else if (winMove == 2) {
                    rvPlayerPrintResult(player2, 1);
                    rvPlayerPrintResult(player1, 0);
                }
                // Draw
                else {
                    rvPlayerPrintResult(player1, 2);
                    rvPlayerPrintResult(player2, 2);
                }

                break;
            }
        }
    }
    

    // -- CLEAN UP MEMORY --
    rvGridDelete(grid);
    rvPlayerDelete(player1);
    rvPlayerDelete(player2);
    
    return 0;
}
