/* Project 8 by David Taralla - PART 3 | 18-04-2010
*  =================================================
*  This is the implementation of rvPlayer.h. See this
*  header for further information on non-static
*  functions which are implemented here.
*  Some non-static functions use other functions
*  which are locale to this file only. Each precedes
*  the first function that uses them.
*/

#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include "rvPlayer.h"
#include "rvGrid.h"


/******************************************************************************
*--------------------------------- New Types ---------------------------------*
******************************************************************************/
struct rvPlayer_t {
    int gridWidth;
    int gridHeight;

    FILE* output;
};

/******************************************************************************
*--------------------------------- Functions ---------------------------------*
******************************************************************************/
rvPlayer* rvPlayerNew(int port, int width, int height) {
    rvPlayer* toReturn = malloc(sizeof(rvPlayer));
    if (toReturn == NULL)
        return NULL;
    
    toReturn->output = stdout;
    toReturn->gridWidth = width;
    toReturn->gridHeight = height;
    
    // To avoid compiler warnings
    port = port;
    
    return toReturn;
}

void rvPlayerDelete(rvPlayer* player) {
    free(player);
}

/* ----------------------------------------------------------------------------
*  Description :
*  ==============
*  Empty the stdin buffer completely.
*  --------------------------------------------------------------------------*/
static void cleanStdinBuffer(void) {
    char c;    
    do
        c = getchar();
    while (c != '\n' && c != EOF);
}

int rvPlayerAddDisk(rvPlayer* player, int* row, int* column) {
    if (player == NULL || row == NULL || column == NULL)
        return -1;
    
    fprintf(player->output, "Where do you want to place your disk ?\nRow: ");
    int scanReturn = scanf("%d", row);
    
    // While wrong input, re-asking row
    while (scanReturn == 0 || *row >= player->gridWidth || *row < 0) {
        fprintf(player->output, "You didn't enter valid data, try again.\n"
                                "Row: ");
        cleanStdinBuffer();
        scanReturn = scanf("%d", row);
    }
    
    fprintf(player->output, "Column: ");
    scanReturn = scanf("%d", column);
    
    // While wrong input, re-asking column
    while (scanReturn == 0 || *column >= player->gridHeight || *column < 0) {
        fprintf(player->output, "You didn't enter valid data, try again.\n"
                                "Column: ");
        cleanStdinBuffer();
        scanReturn = scanf("%d", column);
    }
    
    return 0;
}

FILE* rvPlayerOutput(rvPlayer* player) {
    return player->output;
}

int rvPlayerPrintResult(rvPlayer* player, int state) {
    if (player == NULL)
        return -1;
    
    switch(state) {
        case 0: // Victorious
            fprintf(player->output, "Congratulations! You won!\n");
            break;
        case 1: // Defeated
            fprintf(player->output, "You loose... Try again!\n");
            break;
        case 2: // Draw
            fprintf(player->output, "It is tie!\n");
            break;
        default: // Error, [state] may only be 0, 1 or 2
            return -1;

    }

    return 0;
}

int rvPlayerGetWidth(rvPlayer* player) {
    if (player == NULL)
        return 0;
    else
        return player->gridWidth;
}

int rvPlayerGetHeight(rvPlayer* player) {
    if (player == NULL)
        return 0;
    else
        return player->gridHeight;
}
